/*
 * Copyright (c) 2004 RenderX, Inc. All Rights Reserved.
 *
 * This software is provided as a sample for evaluation
 * of XEP formatter only. RenderX declines every liability
 * for negative effects of its use for other purposes.
 */


/**
 * This sample illustrates how to manipulate intermediate XEP XML format
 * by applying a custom XSLT transformation to it. Included XSLT
 * stylesheet make-landscape.xsl rearranges logical pages into
 * a landscape-oriented document.
 *
 * @author  Alexander Peshkov
 * @version 1.3, 14 September 2004
 */

public class PostProcessing {
    /**
     * Main function reads source XSL FO document, renders it into itermediate format
     * that is transformed with XSLT and then rendered into the PDF.
     * @param args input and output file names
     */
    public static void main(String[] args) {
        if (args.length < 2) {
            System.out.println("Usage: java PostProcessing <input> <output>\nwhere <output> is PDF file.");
            System.exit(1);
        }

        //XSLT stylesheet
        String stylesheetPath = "make-landscape.xsl";

        // Prepare configuration properties
        java.util.Properties options = new java.util.Properties();
        // Find the configuration file. If there exists a system property
        // named com.renderx.xep.CONFIG, do nothing; otherwise, set an
        // explicit CONFIG property in the constructor.
        // Adjust the path to match your installation.
        if (System.getProperty("com.renderx.xep.CONFIG") == null) {
            options.setProperty("CONFIG", "C:\\XEP\\xep.xml");
	}
        // Create an instance of XEP formatter
        com.renderx.xep.FormatterImpl xep = null;
        try {
            xep = new com.renderx.xep.FormatterImpl(options);
        }
        catch (Exception e) {
            System.out.println("Could not create XEP formatter: " + e.toString());
            System.exit(1);
        }

        try {
            // Get system ID
            String sourceId = null;
            try {
                java.io.File in = new java.io.File(args[0]);
                // java.io.File.toURL() is Java 1.2+, not supported by MS JVM
                sourceId = in.toURL().toString();
            }
            catch (Exception e) {
                System.out.println("Incorrect input file: " + e.getMessage());
                System.exit(1);
            }

            // Create XMLReader
            org.xml.sax.XMLReader xmlreader = null;
            try {
                xmlreader=javax.xml.parsers.SAXParserFactory.newInstance().newSAXParser().getXMLReader();
	    }
	    catch (Exception e) {
                System.out.println("Can't create XMLReader: " + e.getMessage());
                System.exit(1);
	    }

            // Create XEPXMLFilter
            XEPXMLFilter xepfilter = new XEPXMLFilter(xmlreader, xep, sourceId);

            // Create a JAXP SAXSource for the source
            javax.xml.transform.sax.SAXSource source = null;
            try {
                org.xml.sax.InputSource saxsrc = new org.xml.sax.InputSource(sourceId);
                source = new javax.xml.transform.sax.SAXSource(xepfilter, saxsrc);
            }
            catch (Exception e) {
                System.out.println("Source creation failed: " + e.getMessage());
                System.exit(1);
            }

            // Create a transformer object
            javax.xml.transform.Transformer transformer = null;
            try {
                javax.xml.transform.TransformerFactory factory =
                    javax.xml.transform.TransformerFactory.newInstance();

                java.io.File in = new java.io.File(stylesheetPath);
                javax.xml.transform.stream.StreamSource stylesheet =
                    new javax.xml.transform.stream.StreamSource(in);

                transformer = factory.newTransformer(stylesheet);
            }
            catch (Exception e) {
                System.out.println("Transformer creation failed: " + e.toString());
                System.exit(1);
            }

            // Open the output file
            java.io.OutputStream out = null;
            try {
                out = new java.io.BufferedOutputStream(new java.io.FileOutputStream(args[1]));
            }
            catch (Exception e) {
                System.out.println("Could not open output file: " + e.toString());
                System.exit(1);
            }

            // Create a SAXResult
            javax.xml.transform.Result result = null;
            try {
                // Create a PostScript generator (it has ContentHandler interface)
                org.xml.sax.ContentHandler handler = xep.createGenerator(new com.renderx.xep.FOTarget(out, "PDF"));
                result = new javax.xml.transform.sax.SAXResult(handler);
            }
            catch (Exception e) {
                System.out.println("Could not create a SAXResult object: " + e.toString());
                System.exit(1);
            }

             // Perform the transformation
            try {
                try {
                    transformer.transform(source, result);
                }
                catch (Exception e) {
                    System.out.println("Transformation error: " + e.toString());
                }
                finally {
                    out.close();
                    System.exit(1);
                }
            }
            catch (Exception e) {
                System.out.println("Could not close the resulting document: " + e.getMessage());
                System.exit(1);
            }
        }
        finally {
            xep.cleanup();
        }
        // Everything is OK
        System.exit(0);
    }

    /**
     * Wrapper class for XEP formatter.
     */
    static class XEPXMLFilter extends org.xml.sax.helpers.XMLFilterImpl {
	final com.renderx.xep.FormatterImpl xep;
	final String systemId;

	XEPXMLFilter (org.xml.sax.XMLReader xmlreader, com.renderx.xep.FormatterImpl formatter, String id) {
	    super(xmlreader);
	    xep = formatter;
	    systemId = id;
	}

   	public void setContentHandler(org.xml.sax.ContentHandler handler) {
            org.xml.sax.ContentHandler xephandler = null;
	    try {
		xephandler = xep.createContentHandler(systemId, handler);
	    }
	    catch (com.renderx.xep.lib.ConfigurationException e) {
		throw new RuntimeException(e.toString());
	    }
	    super.setContentHandler(xephandler);
	}
    }
}
